#include <stdio.h>
#include "lis.h"

int main(int argc, char* argv[])
{
    LIS_MATRIX        A;
    LIS_VECTOR        b,x,u;
    int               is,ie,my_rank,err;
    int               i,n,iter;
    int               options[LIS_OPTIONS_LEN];
    LIS_SCALAR        params[LIS_PARAMS_LEN];
    LIS_SCALAR        status[LIS_STATUS_LEN];

    lis_initialize(argc, argv, NULL, options, params);
    MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

    n  = 9;
    lis_matrix_create(0,n,LIS_COMM_WORLD,&A);

    lis_matrix_get_range(A,&is,&ie);
    for(i=is;i<ie;i++)
    {
        if( i>0   ) lis_matrix_set(i,i-1,1,A);
        if( i<n-1 ) lis_matrix_set(i,i+1,1,A);
        lis_matrix_set(i,i,2,A);
    }
    lis_matrix_assemble(&A,LIS_MATRIX_CRS);

    lis_vector_create(0,n,LIS_COMM_WORLD,&u);
    lis_vector_duplicate(u,&b);
    lis_vector_duplicate(b,&x);

    lis_vector_set_all(1.0,u);
    lis_vector_set_all(0.0,x);
    lis_matvec(A,u,b);

    lis_solve(A,b,x,params,options,status);

    if( my_rank==0 )
    {
        iter = (int)status[LIS_STATUS_ITER];
        printf("iter = %d\n",iter);
    }
    lis_vector_print(x);

    lis_matrix_destroy(A);
    lis_vector_destroy(b);
    lis_vector_destroy(x);
    lis_vector_destroy(u);

    lis_finalize();
    return 0;
}
