/* Copyright (C) 2004-2005 The SSI Project. All rights reserved.
   http://ssi.is.s.u-tokyo.ac.jp/

   This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.  See the LICENSE file for the precise
   terms and conditions for the use of this software.
*/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <ctype.h>
#include <string.h>
#include <stdarg.h>
#ifdef _OPENMP
	#include <omp.h>
#endif
#include "lislib.h"
#include "lis_io.h"

int lis_input_hb(LIS_MATRIX *A, LIS_VECTOR *b, LIS_VECTOR *x, FILE *file, LIS_Comm comm, int matrix_type, ...)
{
	int				bnr,bnc;
	int				err;
	int				*crow,*ccol;
	va_list			vvlist;

	va_start( vvlist, matrix_type );
	bnr = va_arg(vvlist, int);
	bnc = va_arg(vvlist, int);
	crow = va_arg(vvlist, int *);
	ccol = va_arg(vvlist, int *);
	va_end(vvlist);

	switch( matrix_type )
	{
	case LIS_MATRIX_CRS:
		err = lis_input_hb_crs(A,b,x,file,comm,matrix_type,bnr,bnc,crow,ccol);
		break;
	case LIS_MATRIX_CCS:
		break;
	case LIS_MATRIX_MSR:
		break;
	case LIS_MATRIX_DIA:
		break;
	case LIS_MATRIX_ELL:
		break;
	case LIS_MATRIX_JDS:
		break;
	case LIS_MATRIX_BSR:
		break;
	case LIS_MATRIX_BSC:
		break;
	case LIS_MATRIX_VBR:
		break;
	case LIS_MATRIX_DNS:
		break;
	case LIS_MATRIX_COO:
		break;
	}
	return LIS_SUCCESS;
}

int lis_input_hb_get_fmt(char *buf, int size, int *iter, int *width)
{
	char	tmp[64];
	char	*p, *s, *t;

	strncpy(tmp, buf, size); tmp[size] = '\0';
	for(p=tmp;*p!='\0';p++)     *p = (char)tolower(*p);
	p = strchr(tmp, '(');
	if( p!=NULL )
	{
		s = p+1;
		p = strchr(s, ')'); *p = '\0';
		p = strchr(s, 'i');
		if( p==NULL )
		{
			p = strchr(s, 'e');
			if( p==NULL )
			{
				p = strchr(s, 'd');
				if( p==NULL ) return LIS_FAILS;
			}
			t = strchr(s, '.'); *t = '\0';
		}
		*p = '\0'; 
		*iter  = atoi(s);
		*width = atoi(p+1);
	}
	else
	{
		*iter  = 0;
		*width = 0;
	}
	return LIS_SUCCESS;
}

int lis_input_hb_crs(LIS_MATRIX *A, LIS_VECTOR *b, LIS_VECTOR *x, FILE *file, LIS_Comm comm, int matrix_type,int bnr, int bnc, int *row, int *col)
{
	char			buf[BUFSIZE];
	char			title[128], key[128], mtx[64], fmt[64], dat[128];
	char			*p;
	char			MXTYPE_F,MXTYPE_S,MXTYPE_T;
	char			RHSTYP_F,RHSTYP_S,RHSTYP_T;
	int				TOTCRD,PTRCRD,INDCRD,VALCRD,RHSCRD;
	int				NROW,NCOL,NNZERO,NELTVL;
	int				NRHS,NRHSIX;
	int				iptr,iind,ival,irhs;
	int				wptr,wind,wval,wrhs;
	int				nr,nc,nnz,local_nnz;
	int				i,k,j,my_rank;
	int				err;
	int				mmtype;
	int				n,is,ie;
	int				ridx,cidx;
	int				*ptr, *index;
	int				*work;
	int				isb,isx;
	LIS_SCALAR		val;
	LIS_SCALAR		*value;
	LIS_MATRIX		Ain;

#ifdef USE_MPI
    MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);
#else
	my_rank = 0;
#endif
	/* Line 1 */
	if( fgets(buf, BUFSIZE, file) == NULL )
	{
		SETERR("file i/o error\n");
		return LIS_FAILS;
	}
	strncpy(title, buf    ,72); title[72] = '\0';
	strncpy(key  ,&buf[72], 8); key[8]    = '\0';
	printf("title: %s\n",title);
	printf("key  : %s\n",key);

	/* Line 2 */
	if( fgets(buf, BUFSIZE, file) == NULL )
	{
		SETERR("file i/o error\n");
		return LIS_FAILS;
	}
	if( sscanf(buf, "%14d%14d%14d%14d%14d", &TOTCRD, &PTRCRD, &INDCRD, &VALCRD, &RHSCRD) != 5 )
	{
		SETERR("file I/O error\n");
		return LIS_FAILS;
	}
	printf("%14d%14d%14d%14d%14d\n",TOTCRD, PTRCRD, INDCRD, VALCRD, RHSCRD);

	/* Line 3 */
	if( fgets(buf, BUFSIZE, file) == NULL )
	{
		SETERR("file i/o error\n");
		return LIS_FAILS;
	}
	if( sscanf(buf, "%s %d %d %d %d", mtx, &NROW, &NCOL, &NNZERO, &NELTVL) != 5 )
	{
		SETERR("file I/O error\n");
		return LIS_FAILS;
	}
	for(p=mtx;*p!='\0';p++)     *p = (char)tolower(*p);
	MXTYPE_F = mtx[0];
	MXTYPE_S = mtx[1];
	MXTYPE_T = mtx[2];
	if( mtx[0]!='r' )
	{
		SETERR("Not real\n");
		return LIS_FAILS;
	}
	if( mtx[1]!='u' )
	{
		SETERR("Not unsymmetric\n");
		return LIS_FAILS;
	}
	if( mtx[2]!='a' )
	{
		SETERR("Not assembled\n");
		return LIS_FAILS;
	}
	if( NROW!=NCOL )
	{
		SETERR("matrix is not square\n");
		return LIS_FAILS;
	}
	printf("%c%c%c %d %d %d %d\n",MXTYPE_F, MXTYPE_S, MXTYPE_T, NROW, NCOL, NNZERO, NELTVL);

	/* Line 4 */
	if( fgets(buf, BUFSIZE, file) == NULL )
	{
		SETERR("file i/o error\n");
		return LIS_FAILS;
	}
	lis_input_hb_get_fmt( buf    ,16,&iptr,&wptr);
	lis_input_hb_get_fmt(&buf[16],16,&iind,&wind);
	lis_input_hb_get_fmt(&buf[32],20,&ival,&wval);
	lis_input_hb_get_fmt(&buf[52],20,&irhs,&wrhs);
	printf("%d %d %d %d\n",iptr,iind,ival,irhs);
	printf("%d %d %d %d\n",wptr,wind,wval,wrhs);

	/* Line 5 */
	if( RHSCRD!=0 )
	{
		if( fgets(buf, BUFSIZE, file) == NULL )
		{
			SETERR("file i/o error\n");
			return LIS_FAILS;
		}
		sscanf(buf, "%s %d %d", mtx, &NRHS, &NRHSIX);
/*
		if( sscanf(buf, "%s %d %d", mtx, &NRHS, &NRHSIX) != 3 )
		{
			SETERR("file I/O error\n");
			return LIS_FAILS;
		}
*/
		for(p=mtx;*p!='\0';p++)     *p = (char)tolower(*p);
		RHSTYP_F = mtx[0];
		RHSTYP_S = mtx[1];
		RHSTYP_T = mtx[2];
		printf("%c%c%c %d %d\n",RHSTYP_F, RHSTYP_S, RHSTYP_T, NRHS, NRHSIX);
	}

	err = lis_matrix_create(0,NROW,comm,&Ain);
	if( err )
	{
		SETERR("out of memory\n");
		return LIS_FAILS;
	}
	n = Ain->n;
	lis_matrix_get_range(Ain,&is,&ie);
	ptr   = (int *)lis_malloc( (n+1)*sizeof(int) );
	if( ptr==NULL )
	{
		SETERR("out of memory\n");
		return LIS_OUT_OF_MEMORY;
	}
	index   = (int *)lis_malloc( NNZERO*sizeof(int) );
	if( index==NULL )
	{
		SETERR("out of memory\n");
		return LIS_OUT_OF_MEMORY;
	}
	value   = (LIS_SCALAR *)lis_malloc( NNZERO*sizeof(LIS_SCALAR) );
	if( value==NULL )
	{
		SETERR("out of memory\n");
		return LIS_OUT_OF_MEMORY;
	}


	/* read data */
	k = 0;
	for( i=0; i<PTRCRD; i++ )
	{
		if( fgets(buf, BUFSIZE, file) == NULL )
		{
			SETERR("file i/o error in ptr\n");
			return LIS_FAILS;
		}
		p = buf;
		for(j=0;j<iptr&&k<n+1;j++)
		{
			strncpy(dat, p, wptr); dat[wptr] = '\0';
			ptr[k] = atoi(dat) - 1;
			p += wptr;
			k++;
		}
	}

	k = 0;
	for( i=0; i<INDCRD; i++ )
	{
		if( fgets(buf, BUFSIZE, file) == NULL )
		{
			SETERR("file i/o error in index\n");
			return LIS_FAILS;
		}
		p = buf;
		for(j=0;j<iind&&k<NNZERO;j++)
		{
			strncpy(dat, p, wind); dat[wind] = '\0';
			index[k] = atoi(dat) - 1;
			p += wind;
			k++;
		}
	}

	k = 0;
	for( i=0; i<VALCRD; i++ )
	{
		if( fgets(buf, BUFSIZE, file) == NULL )
		{
			SETERR("file i/o error in value\n");
			return LIS_FAILS;
		}
		p = buf;
		for(j=0;j<ival&&k<NNZERO;j++)
		{
			strncpy(dat, p, wval); dat[wval] = '\0';
			value[k] = atof(dat);
			p += wval;
			k++;
		}
	}

	if( RHSCRD>0 )
	{
		/*
		k = 0;
		for( i=0; i<RHSCRD; i++ )
		{
			if( fgets(buf, BUFSIZE, file) == NULL )
			{
				SETERR("file i/o error\n");
				return LIS_FAILS;
			}
			p = buf;
			for(j=0;j<ival&&k<NNZERO;j++)
			{
				strncpy(dat, p, wval); dat[wval] = '\0';
				value[k] = atof(dat);
				p += wval;
				printf("%e ",value[k]);
				k++;
			}
			printf("\n");
		}
		*/
	}

	err = lis_matrix_set_ccs(NNZERO,ptr,index,value,Ain);
	if( err )
	{
		return err;
	}

	err = lis_matrix_assemble(&Ain,LIS_MATRIX_CCS);
	if( err ) return err;

	if( matrix_type==LIS_MATRIX_CCS )
	{
		*A = Ain;
	}
	else
	{
		err = lis_matrix_convert(Ain,A,matrix_type,bnr,bnc,row,col);
		if( err ) return err;
	}


	return LIS_SUCCESS;
}
