/* Copyright (C) 2004-2005 The SSI Project. All rights reserved.
   http://ssi.is.s.u-tokyo.ac.jp/

   This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.  See the LICENSE file for the precise
   terms and conditions for the use of this software.
*/

#include <stdio.h>
#include "lislib.h"
#include "lis_matrix.h"

int lis_matrix_print(LIS_MATRIX Amat)
{
	if( NULL==Amat ) return LIS_FAILS;
	if( NULL==Amat->value ) return LIS_FAILS;

	switch( Amat->matrix_type )
	{
	case LIS_MATRIX_DNS:
		lis_matrix_print_dns(Amat);
		break;
	case LIS_MATRIX_CRS:
		lis_matrix_print_crs(Amat);
		break;
	case LIS_MATRIX_CCS:
		lis_matrix_print_ccs(Amat);
		break;
	case LIS_MATRIX_MSR:
		lis_matrix_print_msr(Amat);
		break;
	case LIS_MATRIX_COO:
		lis_matrix_print_coo(Amat);
		break;
	case LIS_MATRIX_DIA:
		lis_matrix_print_dia(Amat);
		break;
	case LIS_MATRIX_ELL:
		lis_matrix_print_ell(Amat);
		break;
	case LIS_MATRIX_JDS:
		lis_matrix_print_jds(Amat);
		break;
	case LIS_MATRIX_BSR:
		lis_matrix_print_bsr(Amat);
		break;
	case LIS_MATRIX_VBR:
		lis_matrix_print_vbr(Amat);
		break;
	default:
		break;
	}
	return LIS_SUCCESS;
}

int lis_matrix_print_dns(LIS_MATRIX Amat)
{
	int i,j,n,gn;

	n  = Amat->n;
	gn = Amat->gn;
	printf("A = \n");
	for(i=0; i<n; i++)
	{
		printf("%4d (", i+1);
		for(j=0; j<gn; j++)
		{
			printf("%6.2f ", Amat->value[j*n+i]);
		}
		printf(")\n");
	}
	return LIS_SUCCESS;
}

int lis_matrix_print_crs(LIS_MATRIX Amat)
{
	int i,j,k,n;

	n    = Amat->n;
	printf("A = \n");

	k = Amat->ptr[0];
	for(i=0; i<n; i++)
	{
		printf("%4d (", i+1);
		for(j=0; j<n; j++)
		{
			if( Amat->ptr[i+1] > k && Amat->index[k] == j )
			{
				printf("%6.2f ", Amat->value[k]);
				k++;
//				fprintf(file, ch, A.val[k]);
			}
			else
			{
				printf("%6.2f ", 0.0);
//				fprintf(file, ch, 0);
			}
		}
		printf(")\n");
	}
	printf("\n");

	return LIS_SUCCESS;
}

int lis_matrix_print_ccs(LIS_MATRIX Amat)
{
	int i,j,k,n;

	n    = Amat->n;
	printf("A = \n");

	for(i=0; i<n; i++)
	{
		printf("%4d (", i+1);
		for(j=0; j<n; j++)
		{
			for(k=Amat->ptr[j];k<Amat->ptr[j+1];k++)
			{
				if( Amat->index[k] == i )
				{
					printf("%6.2f ", Amat->value[k]);
					break;
				}
			}
			if( k==Amat->ptr[j+1] )
			{
				printf("%6.2f ", 0.0);
			}
		}
		printf(")\n");
	}
	printf("\n");

	return LIS_SUCCESS;
}

int lis_matrix_print_msr(LIS_MATRIX Amat)
{
	int i,j,k,n;

	n    = Amat->n;
	printf("A = \n");

	k = Amat->index[0];
	for(i=0; i<n; i++)
	{
		printf("%4d (", i+1);
		for(j=0; j<n; j++)
		{
			if( i==j )
			{
				printf("%6.2f ", Amat->value[j]);
			}
			else
			{
				if( Amat->index[i+1] > k && Amat->index[k] == j )
				{
					printf("%6.2f ", Amat->value[k]);
					k++;
				}
				else
				{
					printf("%6.2f ", 0.0);
				}
			}
		}
		printf(")\n");
	}
	printf("\n");

	return LIS_SUCCESS;
}

int lis_matrix_print_coo(LIS_MATRIX Amat)
{
	int i,j,k,n;

	n    = Amat->n;
	printf("A = \n");
	for(i=0; i<n; i++)
	{
		printf("%4d (", i+1);
		for(j=0; j<n; j++)
		{
			for(k=0;k<Amat->nnz;k++)
			{
				if( Amat->row[k]==i && Amat->col[k]==j )
				{
					printf("%6.2f ", Amat->value[k]);
					break;
				}
			}
			if( k==Amat->nnz )
			{
				printf("%6.2f ", 0.0);
			}
		}
		printf(")\n");
	}
	return LIS_SUCCESS;
}

int lis_matrix_print_dia(LIS_MATRIX Amat)
{
	int i,j,k,n;

	n    = Amat->n;
	printf("A = \n");
	for(i=0; i<n; i++)
	{
		printf("%4d (", i+1);
		for(j=0; j<n; j++)
		{
			for(k=0;k<Amat->nnd;k++)
			{
				if( Amat->index[k]==j-i )
				{
					break;
				}
			}
			if( k < Amat->nnd )
			{
				printf("%6.2f ", Amat->value[k*n+i]);
			}
			else
			{
				printf("%6.2f ", 0.0);
			}
		}
		printf(")\n");
	}
	return LIS_SUCCESS;
}

int lis_matrix_print_ell(LIS_MATRIX Amat)
{
	int i,j,k,n;

	n    = Amat->n;
	printf("A = \n");
	for(i=0; i<n; i++)
	{
		printf("%4d (", i+1);
		k=0;
		for(j=0; j<n; j++)
		{
			if( Amat->index[k*n + i]==j )
			{
				printf("%6.2f ", Amat->value[k*n + i]);
				k++;
			}
			else
			{
				printf("%6.2f ", 0.0);
			}
		}
		printf(")\n");
	}
	return LIS_SUCCESS;
}

int lis_matrix_print_jds(LIS_MATRIX Amat)
{
	int i,j,k,l,n;

	n    = Amat->n;
	printf("A = \n");
	for(i=0; i<n; i++)
	{
		printf("%4d (", i+1);
		l = 0;
		while( i!=Amat->row[l] ) l++;
		k = 0;
		for(j=0; j<n; j++)
		{
			if( Amat->ptr[k+1]-Amat->ptr[k] >= l 
				&& Amat->index[Amat->ptr[k]+l]==j)
			{
				printf("%6.2f ", Amat->value[Amat->ptr[k]+l]);
				k++;
			}
			else
			{
				printf("%6.2f ", 0.0);
			}
		}
		printf(")\n");
	}
	return LIS_SUCCESS;
}

int lis_matrix_print_bsr(LIS_MATRIX Amat)
{
	int i,j,k,n,bn,nr,bi,ii,bbj,bj,jj,jv;

	n    = Amat->n;
	bn   = Amat->bn;
	nr   = 1 + (n-1)/bn;
	printf("A = \n");

	k = 0;
	for(i=0; i<n; i++)
	{
		bi   = i/bn;
		ii   = i%bn;
		bbj  = -1;
		printf("%4d (", i+1);
		for(j=Amat->bptr[bi]; j<Amat->bptr[bi+1]; j++)
		{
			bj  = Amat->bindex[j];
			jv  = j * bn * bn;
			if( bj-bbj>1 )
			{
				for(jj=0;jj<(bj-bbj-1)*bn;jj++)
				{
					printf("%6.2f ", 0.0);
				}
			}
			for(jj=0;jj<bn;jj++)
			{
				printf("%6.2f ", Amat->value[jv+ii+jj*bn]);
			}
			bbj = bj;
		}
		if( bj!=nr-1 )
		{
			for(jj=0;jj<(nr-bj-1)*bn;jj++)
			{
				printf("%6.2f ", 0.0);
			}
		}
		printf(")\n");
	}
	printf("\n");

	return LIS_SUCCESS;
}

int lis_matrix_print_vbr(LIS_MATRIX Amat)
{
	int i,j,k,n,br,bc,bci,bciold,bri,brw;

	n    = Amat->n;
	printf("A = \n");

	br  = 0;
	bri = 0;
	brw = Amat->row[br+1]-Amat->row[br];
	for(i=0; i<n; i++)
	{
		printf("%4d (", i+1);
		if( i == Amat->row[br+1] )
		{
			br++;
			bri = 0;
			brw = Amat->row[br+1]-Amat->row[br];
		}
		bciold = Amat->bindex[Amat->bptr[br]]-1;
		if( bciold!=-1 )
		{
			for(k=0;k<Amat->col[bciold+1];k++)
			{
				printf("%6.2f ", 0.0);
			}
		}
		for(bc=Amat->bptr[br];bc<Amat->bptr[br+1]; bc++)
		{
			bci = Amat->bindex[bc];
			j   = Amat->col[bci];
			if( bci-1!=bciold )
			{
				for(k=0;k<Amat->col[bci]-Amat->col[bciold+1];k++)
				{
					printf("%6.2f ", 0.0);
				}
			}
			for(k=0;k<Amat->col[bci+1]-Amat->col[bci];k++)
			{
				printf("%6.2f ", Amat->value[Amat->ptr[bc]+k*brw+bri]);
			}
			bciold = bci;
		}
		if( bciold!=Amat->bnc )
		{
			for(k=Amat->col[bciold+1];k<n;k++)
			{
				printf("%6.2f ", 0.0);
			}
		}
		bri++;
		printf(")\n");
	}
	printf("\n");

	return LIS_SUCCESS;
}

int lis_debug_matrix_print(LIS_SCALAR *Amat, int n, int m)
{
	int i,j;

	printf("A = \n");
	for(i=0; i<n; i++)
	{
		printf("%4d (", i);
		for(j=0; j<m; j++)
		{
			printf("%6.2f ", Amat[i*m+j]);
		}
		printf(")\n");
	}
	return LIS_SUCCESS;
}

void lis_vector_print(LIS_VECTOR x)
{
#ifdef USE_MPI
	int i,is,n,k,nprocs,my_rank;

	nprocs  = x->nprocs;
	my_rank = x->my_rank;
	n       = x->n;
	is      = x->is;

	for(k=0;k<nprocs;k++)
	{
		if( k==my_rank )
		{
			for(i=0;i<n;i++)
			{
				printf("%6d  %e\n",i+is,x->value[i]);
			}
		}
		MPI_Barrier(x->comm);
	}
#else
	int i,n;

	n = x->n;

	for(i=0; i<n; i++)
	{
		printf("%6d  %e\n",i,x->value[i]);
	}
#endif
}

void lis_vectori_print(int n, int x[], char *str)
{
	int i;

	printf("%s = (",str);
	for(i=0; i<n; i++)
	{
		printf("%3d ", x[i]);
	}
	printf(")\n");
}
